#ifndef MSP_CORE_APPLICATION_H_
#define MSP_CORE_APPLICATION_H_

namespace Msp {

/**
Base class for applications.  See also RegisteredApplication.
*/
class Application
{
protected:
	class Starter
	{
	protected:
		Starter();
	public:
		virtual ~Starter() { }

		virtual Application *create_app(int, char **) = 0;
	};

	bool done;
	int exit_code;

private:
	static Starter *starter_;
	static Application *app_;
	static void *data_;

	Application(const Application &);
	Application &operator=(const Application &);
protected:
	Application();
public:
	virtual ~Application() { }

	static int run(int, char **, void * =0);
	static void *get_data() { return data_; }

protected:
	virtual int main();
	void catch_signal(int);
	void exit(int);
	virtual void tick() { }
	virtual void sighandler(int) { }
private:
	static void sighandler_(int);
};


/**
Registers the class to be used for program startup.  The main application class
should be derived from this.
*/
template<typename T>
class RegisteredApplication: public Application
{
private:
	class Starter: public Application::Starter
	{
	public:
		Application *create_app(int argc, char **argv) { return new T(argc, argv); }
	};

	static Starter starter_;

protected:
	// Force the starter into existence
	RegisteredApplication() { (void)starter_; }
};

template<typename T>
typename RegisteredApplication<T>::Starter RegisteredApplication<T>::starter_;

} // namespace Msp

#endif
